<?php

namespace Litecms\Placement\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Litecms\Placement\Http\Requests\PlacementRequest;
use Litecms\Placement\Interfaces\PlacementRepositoryInterface;
use Litecms\Placement\Models\Placement;

/**
 * Resource controller class for placement.
 */
class PlacementResourceController extends BaseController
{

    /**
     * Initialize placement resource controller.
     *
     * @param type PlacementRepositoryInterface $placement
     *
     * @return null
     */
    public function __construct(PlacementRepositoryInterface $placement)
    {
        parent::__construct();
        $this->repository = $placement;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Litecms\Placement\Repositories\Criteria\PlacementResourceCriteria::class);
    }

    /**
     * Display a list of placement.
     *
     * @return Response
     */
    public function index(PlacementRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Litecms\Placement\Repositories\Presenter\PlacementPresenter::class)
                ->$function();
        }

        $placements = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('placement::placement.names'))
            ->view('placement::placement.index', true)
            ->data(compact('placements', 'view'))
            ->output();
    }

    /**
     * Display placement.
     *
     * @param Request $request
     * @param Model   $placement
     *
     * @return Response
     */
    public function show(PlacementRequest $request, Placement $placement)
    {

        if ($placement->exists) {
            $view = 'placement::placement.show';
        } else {
            $view = 'placement::placement.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('placement::placement.name'))
            ->data(compact('placement'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new placement.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(PlacementRequest $request)
    {

        $placement = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('placement::placement.name')) 
            ->view('placement::placement.create', true) 
            ->data(compact('placement'))
            ->output();
    }

    /**
     * Create new placement.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(PlacementRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $placement                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('placement::placement.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('placement/placement/' . $placement->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/placement/placement'))
                ->redirect();
        }

    }

    /**
     * Show placement for editing.
     *
     * @param Request $request
     * @param Model   $placement
     *
     * @return Response
     */
    public function edit(PlacementRequest $request, Placement $placement)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('placement::placement.name'))
            ->view('placement::placement.edit', true)
            ->data(compact('placement'))
            ->output();
    }

    /**
     * Update the placement.
     *
     * @param Request $request
     * @param Model   $placement
     *
     * @return Response
     */
    public function update(PlacementRequest $request, Placement $placement)
    {
        try {
            $attributes = $request->all();

            $placement->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('placement::placement.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('placement/placement/' . $placement->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('placement/placement/' . $placement->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the placement.
     *
     * @param Model   $placement
     *
     * @return Response
     */
    public function destroy(PlacementRequest $request, Placement $placement)
    {
        try {

            $placement->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('placement::placement.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('placement/placement/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('placement/placement/' . $placement->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple placement.
     *
     * @param Model   $placement
     *
     * @return Response
     */
    public function delete(PlacementRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('placement::placement.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('placement/placement'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/placement/placement'))
                ->redirect();
        }

    }

    /**
     * Restore deleted placements.
     *
     * @param Model   $placement
     *
     * @return Response
     */
    public function restore(PlacementRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('placement::placement.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/placement/placement'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/placement/placement/'))
                ->redirect();
        }

    }

}
